@@ -57,8 +57,7 @@ def kmeans(samples, num_clusters, num_iters = 10, use_cosine_sim = False):
         if use_cosine_sim:
             dists = samples @ means.t()
         else:
-            diffs = rearrange(samples, 'n d -> n () d') \
-                    - rearrange(means, 'c d -> () c d')
+            diffs = rearrange(samples, 'n d -> n () d') - rearrange(means, 'c d -> () c d')
             dists = -(diffs ** 2).sum(dim = -1)
 
         buckets = dists.max(dim = -1).indices
@@ -157,6 +156,8 @@ class EuclideanCodebook(nn.Module):
 
     @autocast(enabled = False)
     def forward(self, x):
+        x = x.float()
+
         shape, dtype = x.shape, x.dtype
         flatten = rearrange(x, '... d -> (...) d')
 
@@ -263,7 +264,10 @@ class CosineSimCodebook(nn.Module):
 
     @autocast(enabled = False)
     def forward(self, x):
+        x = x.float()
+
         shape, dtype = x.shape, x.dtype
+
         flatten = rearrange(x, '... d -> (...) d')
         flatten = l2norm(flatten)
 
@@ -307,8 +311,8 @@ class VectorQuantize(nn.Module):
         self,
         dim,
         codebook_size,
-        n_embed = None,
         codebook_dim = None,
+        heads = 1,
         decay = 0.8,
         eps = 1e-5,
         kmeans_init = False,
@@ -317,8 +321,7 @@ class VectorQuantize(nn.Module):
         threshold_ema_dead_code = 0,
         channel_last = True,
         accept_image_fmap = False,
-        commitment_weight = None,
-        commitment = 1., # deprecate in next version, turn off by default
+        commitment_weight = 1.,
         orthogonal_reg_weight = 0.,
         orthogonal_reg_active_codes_only = False,
         orthogonal_reg_max_codes = None,
@@ -326,29 +329,27 @@ class VectorQuantize(nn.Module):
         sync_codebook = False
     ):
         super().__init__()
-        n_embed = default(n_embed, codebook_size)
-
+        self.heads = heads
         codebook_dim = default(codebook_dim, dim)
-        requires_projection = codebook_dim != dim
-        self.project_in = nn.Linear(dim, codebook_dim) if requires_projection \
-                          else nn.Identity()
-        self.project_out = nn.Linear(codebook_dim, dim) if requires_projection \
-                           else nn.Identity()
+        codebook_input_dim = codebook_dim * heads
+
+        requires_projection = codebook_input_dim != dim
+        self.project_in = nn.Linear(dim, codebook_input_dim) if requires_projection else nn.Identity()
+        self.project_out = nn.Linear(codebook_input_dim, dim) if requires_projection else nn.Identity()
 
         self.eps = eps
-        self.commitment_weight = default(commitment_weight, commitment)
+        self.commitment_weight = commitment_weight
 
         has_codebook_orthogonal_loss = orthogonal_reg_weight > 0
         self.orthogonal_reg_weight = orthogonal_reg_weight
         self.orthogonal_reg_active_codes_only = orthogonal_reg_active_codes_only
         self.orthogonal_reg_max_codes = orthogonal_reg_max_codes
 
-        codebook_class = EuclideanCodebook if not use_cosine_sim \
-                         else CosineSimCodebook
+        codebook_class = EuclideanCodebook if not use_cosine_sim else CosineSimCodebook
 
         self._codebook = codebook_class(
             dim = codebook_dim,
-            codebook_size = n_embed,
+            codebook_size = codebook_size,
             kmeans_init = kmeans_init,
             kmeans_iters = kmeans_iters,
             decay = decay,
@@ -369,7 +370,7 @@ class VectorQuantize(nn.Module):
         return self._codebook.embed
 
     def forward(self, x):
-        shape, device, codebook_size = x.shape, x.device, self.codebook_size
+        shape, device, heads, is_multiheaded, codebook_size = x.shape, x.device, self.heads, self.heads > 1, self.codebook_size
 
         need_transpose = not self.channel_last and not self.accept_image_fmap
 
@@ -382,6 +383,9 @@ class VectorQuantize(nn.Module):
 
         x = self.project_in(x)
 
+        if is_multiheaded:
+            x = rearrange(x, 'b n (h d) -> (b h) n d', h = heads)
+
         quantize, embed_ind = self._codebook(x)
 
         if self.training:
@@ -410,6 +414,10 @@ class VectorQuantize(nn.Module):
                 orthogonal_reg_loss = orthgonal_loss_fn(codebook)
                 loss = loss + orthogonal_reg_loss * self.orthogonal_reg_weight
 
+        if is_multiheaded:
+            quantize = rearrange(quantize, '(b h) n d -> b n (h d)', h = heads)
+            embed_ind = rearrange(embed_ind, '(b h) n -> b n h', h = heads)
+
         quantize = self.project_out(quantize)
 
         if need_transpose:
@@ -417,6 +425,6 @@ class VectorQuantize(nn.Module):
 
         if self.accept_image_fmap:
             quantize = rearrange(quantize, 'b (h w) c -> b c h w', h = height, w = width)
-            embed_ind = rearrange(embed_ind, 'b (h w) -> b h w', h = height, w = width)
+            embed_ind = rearrange(embed_ind, 'b (h w) ... -> b h w ...', h = height, w = width)
 
         return quantize, embed_ind, loss
